/*
 *  xfmedia - simple gtk2 media player based on xine
 *
 *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <gtk/gtk.h>
#if !GTK_CHECK_VERSION(2, 4, 0)
#include "my-gtkexpander.h"
#define gtk_event_box_set_visible_window(x, y)
#define g_queue_get_length(x) (x->length)
#endif
#include <gdk-pixbuf/gdk-pixdata.h>

#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>

#include "exo/exo-ellipsized-label.h"

#include "gtkxine.h"
#include "mainwin.h"
#include "playlist.h"
#include <xfmedia/xfmedia-settings.h>
#include "trayicon.h"
#include "keybindings.h"
#include "main.h"
#include "jumptofilewin.h"
#include "xfmedia-internal.h"
#include "mediamarks.h"

#define MAX_SHUFFLE_QUEUE_DEPTH 100
#define AUTOHIDE_TIMEOUT        500

gboolean
xfmedia_mainwin_do_autohide(XfMediaMainwin *mwin)
{
    gboolean do_it = FALSE;
    GdkScreen *gscreen;
    gint scr_w, scr_h, x, y, w, h, a_x, a_y, a_w, a_h;
    GdkRectangle extents;
    
    gscreen = gtk_widget_get_screen(mwin->window);
    scr_w = gdk_screen_get_width(gscreen);
    scr_h = gdk_screen_get_height(gscreen);
    
    gdk_window_get_frame_extents(mwin->window->window, &extents);
    x = extents.x;
    y = extents.y;
    w = extents.width;
    h = extents.height;
    
    if(y == 0) {
        a_x = x;
        a_y = 0;
        a_w = w;
        a_h = 3;
        do_it = TRUE;
    } else if(y == scr_h - h) {
        a_x = x;
        a_y = scr_h - 3;
        a_w = w;
        a_h = 3;
        do_it = TRUE;
    } else if(x == 0) {
        a_x = 0;
        a_y = y;
        a_w = 3;
        a_h = h;
        gtk_window_move(GTK_WINDOW(mwin->autohide_window), 0, y);
        do_it = TRUE;
    } else if(x == scr_w - w) {
        a_x = scr_w - 3;
        a_y = y;
        a_w = 3;
        a_h = h;
        do_it = TRUE;
    }
    
    if(do_it) {
        gtk_widget_set_size_request(mwin->autohide_window, a_w, a_h);
        gtk_widget_realize(mwin->autohide_window);
        gtk_window_resize(GTK_WINDOW(mwin->autohide_window), a_w, a_h);
        gtk_window_move(GTK_WINDOW(mwin->autohide_window), a_x, a_y);
        if(xfmedia_settings_get_bool("/xfmedia/general/window_sticky"))
            gtk_window_stick(GTK_WINDOW(mwin->autohide_window));
        else
            gtk_window_unstick(GTK_WINDOW(mwin->autohide_window));
        gtk_window_set_keep_above(GTK_WINDOW(mwin->autohide_window), TRUE);
        gtk_window_set_skip_taskbar_hint(GTK_WINDOW(mwin->autohide_window), TRUE);
        gtk_window_set_accept_focus(GTK_WINDOW(mwin->autohide_window), FALSE);
        gtk_widget_show(mwin->autohide_window);
        gtk_widget_hide(mwin->window);
    }
    
    mwin->autohide_timer = 0;
    
    return FALSE;
}

static void
instantiate_extra_xine(XfMediaMainwin *mwin)
{
    mwin->extra_vp = xine_open_video_driver(gtk_xine_get_raw_xine_engine(mwin->gtx),
            "none", XINE_VISUAL_TYPE_NONE, NULL);
    mwin->extra_ap = xine_open_audio_driver(gtk_xine_get_raw_xine_engine(mwin->gtx),
            "none", NULL);
    mwin->extra_stream = xine_stream_new(gtk_xine_get_raw_xine_engine(mwin->gtx),
            mwin->extra_ap,    mwin->extra_vp);
}

static void
create_autohide_window(XfMediaMainwin *mwin)
{
    GtkWidget *win;
    guint opacity;
    gint w, h;
    
    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(win), "xfmedia-autohide-window");
    gtk_window_set_decorated(GTK_WINDOW(win), FALSE);
    gtk_window_set_keep_above(GTK_WINDOW(win), TRUE);
    gtk_window_set_skip_taskbar_hint(GTK_WINDOW(win), TRUE);
    gtk_window_set_skip_pager_hint(GTK_WINDOW(win), TRUE);
    g_signal_connect(G_OBJECT(win), "enter-notify-event",
            G_CALLBACK(xfmedia_autohidewin_enter_notify), mwin);
    
    gtk_window_get_size(GTK_WINDOW(mwin->window), &w, &h);
    gtk_widget_set_size_request(win, w, h);
    
    /* set 40% transparency (if supported) */
    opacity = (guint)(0.6*0xffffffff);
    gdk_property_change(mwin->fs_controls->window,
            gdk_atom_intern("_NET_WM_WINDOW_OPACITY", FALSE),
            gdk_atom_intern("CARDINAL", FALSE), 32,
            GDK_PROP_MODE_REPLACE, (const guchar *)&opacity, sizeof(guint));
    
    mwin->autohide_window = win;
}

static void
create_fs_control_window(XfMediaMainwin *mwin)
{
    GtkWidget *tophbox, *img, *btn, *lbl, *slider, *evtbox;
    gint iw, ih;
    guint opacity;
    
    mwin->fs_controls = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(mwin->fs_controls),
            "xfmedia-fullscreen-control-window");
    gtk_window_set_decorated(GTK_WINDOW(mwin->fs_controls), FALSE);
    gtk_window_set_keep_above(GTK_WINDOW(mwin->fs_controls), TRUE);
    gtk_container_set_border_width(GTK_CONTAINER(mwin->fs_controls), BORDER/2);
    
    tophbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(tophbox);
    gtk_container_add(GTK_CONTAINER(mwin->fs_controls), tophbox);
    
    gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &iw, &ih);
    
    img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_PREVIOUS,
            GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(img);
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(tophbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_prev_cb), mwin);
    
    mwin->fs_play_btn_img = img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_PLAY,
            GTK_ICON_SIZE_BUTTON);
    g_object_ref(G_OBJECT(img));
    gtk_widget_show(img);
    mwin->fs_playpause_btn = btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), mwin->fs_play_btn_img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(tophbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_playpause_cb), mwin);
    
    mwin->fs_pause_btn_img = img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_PAUSE,
            GTK_ICON_SIZE_BUTTON);
    g_object_ref(G_OBJECT(img));
    gtk_widget_show(img);
    
    img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_STOP,
            GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(img);
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(tophbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_stop_cb), mwin);
    
    img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_NEXT,
            GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(img);
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(tophbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_next_cb), mwin);
    
    evtbox = gtk_event_box_new();
    gtk_event_box_set_visible_window(GTK_EVENT_BOX(evtbox), FALSE);
    gtk_widget_show(evtbox);
    gtk_box_pack_start(GTK_BOX(tophbox), evtbox, FALSE, FALSE, 0);
    gtk_widget_add_events(evtbox, GDK_BUTTON_PRESS);
    g_signal_connect(G_OBJECT(evtbox), "button-press-event",
            G_CALLBACK(xfmedia_pos_lbl_click_cb), mwin);
    
    mwin->fs_time_label = lbl = gtk_label_new(NULL);
    gtk_label_set_markup(GTK_LABEL(lbl),
            "<span font_family=\"monospace\"> 00:00</span>");
    gtk_widget_show(lbl);
    gtk_container_add(GTK_CONTAINER(evtbox), lbl);
    
    mwin->fs_position_slider = slider = gtk_hscale_new(NULL);
    gtk_scale_set_draw_value(GTK_SCALE(slider), FALSE);
    gtk_range_set_increments(GTK_RANGE(slider), 1, 10);
    gtk_range_set_update_policy(GTK_RANGE(slider), GTK_UPDATE_DISCONTINUOUS);
    gtk_widget_show(slider);
    gtk_box_pack_start(GTK_BOX(tophbox), slider, TRUE, TRUE, 0);
    g_signal_connect(G_OBJECT(slider), "value-changed",
            G_CALLBACK(xfmedia_position_slider_cb), mwin);
    g_signal_connect(G_OBJECT(slider), "button-press-event",
            G_CALLBACK(xfmedia_pos_slider_btn_press_cb), mwin);
    g_signal_connect(G_OBJECT(slider), "button-release-event",
            G_CALLBACK(xfmedia_pos_slider_btn_rls_cb), mwin);
    
    gtk_widget_realize(mwin->fs_controls);
    
    /* set 40% transparency (if supported) */
    opacity = (guint)(0.6*0xffffffff);
    gdk_property_change(mwin->fs_controls->window,
            gdk_atom_intern("_NET_WM_WINDOW_OPACITY", FALSE),
            gdk_atom_intern("CARDINAL", FALSE), 32,
            GDK_PROP_MODE_REPLACE, (const guchar *)&opacity, sizeof(guint));
}

XfMediaMainwin *
xfmedia_mainwin_new(GtkXine *gtx, GtkWindow *video_window)
{
    XfMediaMainwin *mwin;
    GtkWidget *topwin, *topvbox, *playlist_pane, *mm_menu;
    GtkWidget *hbox, *slider, *vbox, *btn, *lbl, *img, *menu, *mi, *evtbox;
    GdkPixbuf *winicon;
    gint iw, ih, w, h_s, h_l;
    GtkTooltips *ttips;
    gchar *user_kb_file;
    
    g_return_val_if_fail(gtx != NULL, NULL);
    
    mwin = g_new0(XfMediaMainwin, 1);
    
    DBG("setting up gtx signals");
    mwin->gtx = gtx;
    g_signal_connect(G_OBJECT(gtx), "button-press-event",
            G_CALLBACK(xfmedia_vwin_button_press_cb), mwin);
    g_signal_connect(G_OBJECT(gtx), "motion-notify-event",
            G_CALLBACK(xfmedia_vwin_motion_notify_cb), mwin);
    g_signal_connect(G_OBJECT(gtx), "stream-ended",
            G_CALLBACK(xfmedia_gtkxine_stream_ended_cb), mwin);
    g_signal_connect(G_OBJECT(gtx), "ui-message",
            G_CALLBACK(xfmedia_gtkxine_ui_message_cb), mwin);
    g_signal_connect(G_OBJECT(gtx), "progress-message",
            G_CALLBACK(xfmedia_gtkxine_progress_message_cb), mwin);
    g_signal_connect(G_OBJECT(gtx), "format-changed",
            G_CALLBACK(xfmedia_gtkxine_format_changed_cb), mwin);
    DBG("  done");
    
    instantiate_extra_xine(mwin);
    
    mwin->video_window = GTK_WIDGET(video_window);
    gtk_widget_add_events(GTK_WIDGET(video_window), GDK_BUTTON_PRESS_MASK|GDK_KEY_PRESS_MASK);
    g_signal_connect(G_OBJECT(video_window), "button-press-event",
            G_CALLBACK(xfmedia_vwin_button_press_cb), mwin);
    g_signal_connect(G_OBJECT(video_window), "motion-notify-event",
            G_CALLBACK(xfmedia_vwin_motion_notify_cb), mwin);
    g_signal_connect(G_OBJECT(video_window), "delete-event",
            G_CALLBACK(xfmedia_vwin_delete_cb), mwin);
    g_signal_connect(G_OBJECT(video_window), "key-press-event",
            G_CALLBACK(xfmedia_mainwin_keypress_cb), mwin);
    g_signal_connect(G_OBJECT(video_window), "map-event",
            G_CALLBACK(xfmedia_vwin_mapunmap_cb), mwin);
    g_signal_connect(G_OBJECT(video_window), "unmap-event",
            G_CALLBACK(xfmedia_vwin_mapunmap_cb), mwin);
    
    mwin->tooltips = ttips = gtk_tooltips_new();
    
    mwin->window = topwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(topwin), "Xfmedia");
    gtk_container_set_border_width(GTK_CONTAINER(topwin), BORDER/2);
    if(xfmedia_settings_get_bool("/xfmedia/playlist/playlist_expanded")) {
        w = xfmedia_settings_get_int("/xfmedia/general/width");
        h_l = xfmedia_settings_get_int("/xfmedia/general/height_large");
        if(h_l != -1 &&  w != -1)
            gtk_window_resize(GTK_WINDOW(topwin), w, h_l);
        else
            gtk_window_resize(GTK_WINDOW(topwin), 475, 550);
    } else {
        w = xfmedia_settings_get_int("/xfmedia/general/width");
        h_s = xfmedia_settings_get_int("/xfmedia/general/height_small");
        if(h_s != -1 && w != -1)
            gtk_window_resize(GTK_WINDOW(topwin), w, h_s);
        else {
            gtk_window_get_size(GTK_WINDOW(topwin), &w, &h_s);
            gtk_window_resize(GTK_WINDOW(topwin), 500, h_s);
        }
    }
    gtk_widget_add_events(topwin, GDK_BUTTON_PRESS|GDK_CONFIGURE|GDK_KEY_PRESS);
    g_signal_connect(G_OBJECT(topwin), "button-press-event",
            G_CALLBACK(xfmedia_mainwin_menu_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "configure-event",
            G_CALLBACK(xfmedia_mainwin_configure_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "window-state-event",
            G_CALLBACK(xfmedia_mainwin_winstate_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "leave-notify-event",
            G_CALLBACK(xfmedia_mainwin_enterleave_notify_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "enter-notify-event",
            G_CALLBACK(xfmedia_mainwin_enterleave_notify_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "map-event",
            G_CALLBACK(xfmedia_mainwin_map_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "unmap-event",
            G_CALLBACK(xfmedia_mainwin_unmap_cb), mwin);
    g_signal_connect(G_OBJECT(topwin), "key-press-event",
            G_CALLBACK(xfmedia_mainwin_keypress_cb), mwin);
    
    xfmedia_keybindings_init();
    user_kb_file = xfce_resource_save_location(XFCE_RESOURCE_CONFIG,
            "xfmedia/keybindings.rc", FALSE);
    if(user_kb_file && g_file_test(user_kb_file, G_FILE_TEST_EXISTS)) {
        xfmedia_keybindings_load(SYSCONFDIR "/xdg/xfmedia/keybindings.rc");
        xfmedia_keybindings_load(user_kb_file);
    } else
        xfmedia_keybindings_load(NULL);
    if(user_kb_file)
        g_free(user_kb_file);
    
    winicon = gdk_pixbuf_from_pixdata(&xfmedia_icon, TRUE, NULL);
    if(winicon) {
        gtk_window_set_default_icon(winicon);
        g_object_unref(G_OBJECT(winicon));
    }
    
    topvbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(topvbox);
    gtk_container_add(GTK_CONTAINER(topwin), topvbox);
    gtk_widget_add_events(topvbox, GDK_BUTTON_PRESS);
    g_signal_connect(G_OBJECT(topvbox), "button-press-event",
            G_CALLBACK(xfmedia_mainwin_menu_cb), mwin);
    
    evtbox = gtk_event_box_new();
    gtk_event_box_set_visible_window(GTK_EVENT_BOX(evtbox), FALSE);
    gtk_widget_show(evtbox);
    gtk_box_pack_start(GTK_BOX(topvbox), evtbox, FALSE, FALSE, 0);
    gtk_widget_add_events(evtbox, GDK_BUTTON_PRESS);
    g_signal_connect(G_OBJECT(evtbox), "button-press-event",
            G_CALLBACK(xfmedia_mainwin_menu_cb), mwin);
            
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_container_add(GTK_CONTAINER(evtbox), hbox);
    
    evtbox = gtk_event_box_new();
    gtk_event_box_set_visible_window(GTK_EVENT_BOX(evtbox), FALSE);
    gtk_widget_show(evtbox);
    gtk_box_pack_start(GTK_BOX(hbox), evtbox, FALSE, FALSE, 0);
    gtk_widget_add_events(evtbox, GDK_BUTTON_PRESS);
    g_signal_connect(G_OBJECT(evtbox), "button-press-event",
            G_CALLBACK(xfmedia_pos_lbl_click_cb), mwin);
    
    mwin->time_label = lbl = gtk_label_new(NULL);
    gtk_label_set_markup(GTK_LABEL(lbl),
            "<span font_family=\"monospace\" size=\"large\" weight=\"bold\"> 00:00</span>");
    gtk_widget_show(lbl);
    gtk_container_add(GTK_CONTAINER(evtbox), lbl);
    
    mwin->song_label = lbl = exo_ellipsized_label_new("");
    exo_ellipsized_label_set_ellipsize(EXO_ELLIPSIZED_LABEL(lbl),
            EXO_PANGO_ELLIPSIZE_END);
    gtk_label_set_markup(GTK_LABEL(lbl), "<span size=\"large\"> </span>");
    gtk_label_set_use_markup(GTK_LABEL(lbl), TRUE);
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, TRUE, TRUE, 0);
    
    evtbox = gtk_event_box_new();
    gtk_event_box_set_visible_window(GTK_EVENT_BOX(evtbox), FALSE);
    gtk_widget_show(evtbox);
    gtk_box_pack_start(GTK_BOX(topvbox), evtbox, FALSE, FALSE, 0);
    gtk_widget_add_events(evtbox, GDK_BUTTON_PRESS);
    g_signal_connect(G_OBJECT(evtbox), "button-press-event",
            G_CALLBACK(xfmedia_mainwin_menu_cb), mwin);
    
    hbox = gtk_hbox_new(FALSE, BORDER-2);
    gtk_widget_show(hbox);
    gtk_container_add(GTK_CONTAINER(evtbox), hbox);
    
    mwin->position_slider = slider = gtk_hscale_new(NULL);
    gtk_scale_set_draw_value(GTK_SCALE(slider), FALSE);
    gtk_range_set_increments(GTK_RANGE(slider), 1, 10);
    gtk_range_set_update_policy(GTK_RANGE(slider), GTK_UPDATE_DISCONTINUOUS);
    gtk_widget_show(slider);
    gtk_box_pack_start(GTK_BOX(hbox), slider, TRUE, TRUE, 0);
    g_signal_connect(G_OBJECT(slider), "value-changed",
            G_CALLBACK(xfmedia_position_slider_cb), mwin);
    g_signal_connect(G_OBJECT(slider), "button-press-event",
            G_CALLBACK(xfmedia_pos_slider_btn_press_cb), mwin);
    g_signal_connect(G_OBJECT(slider), "button-release-event",
            G_CALLBACK(xfmedia_pos_slider_btn_rls_cb), mwin);
    
    gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &iw, &ih);
    
    img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_PREVIOUS,
            GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(img);
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_prev_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Previous Track"), NULL);
    
    mwin->play_btn_img = img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_PLAY,
            GTK_ICON_SIZE_BUTTON);
    g_object_ref(G_OBJECT(img));
    gtk_widget_show(img);
    mwin->playpause_btn = btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_playpause_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Play/Pause Track"), NULL);
    
    mwin->pause_btn_img = img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_PAUSE,
            GTK_ICON_SIZE_BUTTON);
    g_object_ref(G_OBJECT(img));
    gtk_widget_show(img);
    
    img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_STOP,
            GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(img);
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_stop_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Stop Playback"), NULL);
    
    img = xfmedia_squished_stock_icon_load(GTK_STOCK_MEDIA_NEXT,
            GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(img);
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), img);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(xfmedia_next_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Next Track"), NULL);
    
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox);
    hbox = gtk_hbox_new(FALSE, BORDER-4);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    
    mwin->playlist_pane = playlist_pane = gtk_expander_new_with_mnemonic(_("_Playlist"));
    if(xfmedia_settings_get_bool("/xfmedia/playlist/playlist_expanded"))
        gtk_expander_set_expanded(GTK_EXPANDER(playlist_pane), TRUE);
    else
        gtk_expander_set_expanded(GTK_EXPANDER(playlist_pane), FALSE);
    gtk_widget_show(playlist_pane);
    gtk_box_pack_start(GTK_BOX(topvbox), playlist_pane, TRUE, TRUE, 0);
    g_signal_connect_after(G_OBJECT(playlist_pane), "activate",
            G_CALLBACK(xfmedia_mainwin_expander_activate_cb), mwin);
    
    mwin->plist = xfmedia_playlist_new(mwin);
    gtk_widget_show(mwin->plist->playlist_widget);
    gtk_container_add(GTK_CONTAINER(playlist_pane), mwin->plist->playlist_widget);
    
    /* right-click popup menu */
    
    mwin->popup_menu = menu = gtk_menu_new();
    gtk_widget_show(menu);
    
    mi = gtk_image_menu_item_new_with_mnemonic(_("About _Xfmedia"));
    img = gtk_image_new_from_stock(GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
    gtk_widget_show(img);
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect(G_OBJECT(mi), "activate",
            G_CALLBACK(xfmedia_about_cb), mwin);
    
    mi = gtk_separator_menu_item_new();
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    
    img = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
    gtk_widget_show(img);
    mi = gtk_image_menu_item_new_with_mnemonic(_("_Preferences"));
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect_swapped(G_OBJECT(mi), "activate",
            G_CALLBACK(xfmedia_settings_show_dialog), mwin);
    
    mi = gtk_separator_menu_item_new();
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);

#if GTK_CHECK_VERSION(2, 4, 0)  /* GtkTreeModelFilter is gtk 2.4+ only */
    mi = gtk_image_menu_item_new_with_mnemonic(_("_Jump to File..."));
    img = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
    gtk_widget_show(img);
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect_swapped(G_OBJECT(mi), "activate",
            G_CALLBACK(xfmedia_jump_to_file_show), mwin);
#endif
    
    mwin->mmarks_mi = mi = gtk_image_menu_item_new_with_mnemonic(_("_Mediamarks"));
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    mm_menu = xfmedia_mediamarks_create_menu(mwin);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mi), mm_menu);
    xfmedia_mediamarks_hook_changed(xfmedia_mainwin_mediamarks_changed_cb, mwin);
    
    mi = gtk_separator_menu_item_new();
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    
    gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &iw, &ih);
    
    mwin->shuffle_mi = mi = gtk_check_menu_item_new_with_mnemonic(_("_Shuffle"));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi),
            xfmedia_settings_get_bool("/xfmedia/playlist/shuffle"));
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect(G_OBJECT(mi), "toggled",
            G_CALLBACK(xfmedia_shuffle_menu_toggle_cb), mwin);
    
    mwin->repeat_mi = mi = gtk_check_menu_item_new_with_mnemonic(_("_Repeat"));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi),
            xfmedia_settings_get_bool("/xfmedia/playlist/repeat"));
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect(G_OBJECT(mi), "toggled",
            G_CALLBACK(xfmedia_repeat_menu_toggle_cb), mwin);
    
    mi = gtk_check_menu_item_new_with_mnemonic(_("Repeat Single _Track"));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi),
            xfmedia_settings_get_bool("/xfmedia/playlist/repeat_one"));
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect(G_OBJECT(mi), "toggled",
            G_CALLBACK(xfmedia_repeat_one_menu_toggle_cb), mwin);
    
    mwin->svis_mi = mi = gtk_check_menu_item_new_with_mnemonic(_("Show Audio _Visualization"));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi),
            xfmedia_settings_get_bool("/xfmedia/general/show_audio_vis"));
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect(G_OBJECT(mi), "toggled",
            G_CALLBACK(xfmedia_show_vis_menu_toggle_cb), mwin);
    
    mi = gtk_separator_menu_item_new();
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    
    mi = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
    gtk_widget_show(mi);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    g_signal_connect(G_OBJECT(mi), "activate",
            G_CALLBACK(xfmedia_mainwin_quit_mi_cb), mwin);
    
    gtk_widget_realize(topwin);
    
    create_fs_control_window(mwin);
    create_autohide_window(mwin);
    
    if(xfmedia_settings_get_bool("/xfmedia/general/show_tray_icon")) {
        xfmedia_mainwin_create_systray_item(mwin);
        gtk_widget_show(mwin->tray_icon);
    }
    
    /* apply settings */
    
    h_s = xfmedia_settings_get_int("/xfmedia/general/height_small");
    if(xfmedia_settings_get_bool("/xfmedia/playlist/playlist_expanded")) {
        h_l = xfmedia_settings_get_int("/xfmedia/general/height_large");
        if(h_l == -1) {
            gtk_window_get_size(GTK_WINDOW(topwin), &w, &h_l);
            xfmedia_settings_set_int("/xfmedia/general/height_large", h_l);
            xfmedia_settings_set_int("/xfmedia/general/width", w);
        }
    } else if(h_s == -1) {
        gtk_window_get_size(GTK_WINDOW(topwin), &w, &h_s);
        xfmedia_settings_set_int("/xfmedia/general/height_small", h_s);
        xfmedia_settings_set_int("/xfmedia/general/width", w);
    }
    
    if(xfmedia_settings_get_bool("/xfmedia/general/window_sticky"))
        gtk_window_stick(GTK_WINDOW(topwin));
    gtk_window_set_keep_above(GTK_WINDOW(topwin),
            xfmedia_settings_get_bool("/xfmedia/general/window_above"));
    
    mwin->pos_id = g_timeout_add(500, xfmedia_position_cb, mwin);
    
    return mwin;
}

gboolean
xfmedia_mainwin_get_file_info(XfMediaMainwin *mwin, const gchar *filename,
        gchar **name, gint *length)
{
    xine_stream_t *stream;
    
    if(filename) {
        stream = mwin->extra_stream;
        if(!xine_open(stream, filename))
            return FALSE;
    } else
        stream = gtk_xine_get_raw_stream(mwin->gtx);
    
    if(stream) {
        gint pos_stream, pos_time;
        
        *name = xfmedia_sinfo_to_string(stream,
                xfmedia_settings_get_string("/xfmedia/playlist/playlist_format"),
                filename ? filename : gtk_xine_get_current_mrl(mwin->gtx));
        
        /* if this fails, return -2 as the length */
        if(filename) {
            if(xine_get_pos_length(stream, &pos_stream, &pos_time, length))
                *length /= 1000;
            else
                *length = -2;
        } else {
            if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, length))
                *length /= 1000;
            else
                *length = -2;
        }
        
        if(filename)
            xine_close(stream);
    } else
        return FALSE;
    
    return TRUE;
}

void
xfmedia_mainwin_add_file(XfMediaMainwin *mwin, const gchar *filename,
        gint index, gboolean defer_info)
{
    gchar *name = NULL;
    gint length = 0;
    
    if(!defer_info)
        xfmedia_mainwin_get_file_info(mwin, filename, &name, &length);
    
    if(!name)
        name = xfmedia_filename_to_name(filename);
    
    xfmedia_playlist_add_file(mwin->plist, index, name, length, filename, !defer_info);
    g_free(name);
}

void
xfmedia_mainwin_set_song_label(XfMediaMainwin *mwin, const gchar *str)
{
    gchar *str_esc, *str_markup;
    
    str_esc = g_markup_escape_text(str, strlen(str));
    str_markup = g_strdup_printf("<span size=\"large\">%s</span>", str_esc);
    gtk_label_set_markup(GTK_LABEL(mwin->song_label), str_markup);
    
    g_free(str_esc);
    g_free(str_markup);
}

void
xfmedia_mainwin_create_systray_item(XfMediaMainwin *mwin)
{
    GdkScreen *gscreen = gtk_widget_get_screen(mwin->window);
    
    mwin->tray_icon = xfmedia_tray_icon_create(gscreen);
    g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN(mwin->tray_icon))),
            "button-press-event",
            G_CALLBACK(xfmedia_tray_icon_button_press_cb), mwin);
}

void
xfmedia_mainwin_connect_slider_callback(XfMediaMainwin *mwin)
{
    g_signal_handlers_unblock_by_func(G_OBJECT(mwin->position_slider),
            G_CALLBACK(xfmedia_position_slider_cb), mwin);
    g_signal_handlers_unblock_by_func(G_OBJECT(mwin->fs_position_slider),
            G_CALLBACK(xfmedia_position_slider_cb), mwin);
}

void
xfmedia_mainwin_disconnect_slider_callback(XfMediaMainwin *mwin)
{
    g_signal_handlers_block_by_func(G_OBJECT(mwin->position_slider),
            G_CALLBACK(xfmedia_position_slider_cb), mwin);
    g_signal_handlers_block_by_func(G_OBJECT(mwin->fs_position_slider),
            G_CALLBACK(xfmedia_position_slider_cb), mwin);
}

void
xfmedia_mainwin_set_play_buttons(XfMediaMainwin *mwin)
{
    gtk_container_remove(GTK_CONTAINER(mwin->playpause_btn),
            gtk_bin_get_child(GTK_BIN(mwin->playpause_btn)));
    gtk_container_add(GTK_CONTAINER(mwin->playpause_btn),
            mwin->play_btn_img);
    gtk_container_remove(GTK_CONTAINER(mwin->fs_playpause_btn),
            gtk_bin_get_child(GTK_BIN(mwin->fs_playpause_btn)));
    gtk_container_add(GTK_CONTAINER(mwin->fs_playpause_btn),
            mwin->fs_play_btn_img);
    if(mwin->tray_icon_playpause_btn && mwin->tray_icon_play_img) {
        GtkWidget *lbl;
        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mwin->tray_icon_playpause_btn),
                mwin->tray_icon_play_img);
        lbl = gtk_bin_get_child(GTK_BIN(mwin->tray_icon_playpause_btn));
        gtk_label_set_text(GTK_LABEL(lbl), _("_Play"));
        gtk_label_set_use_underline(GTK_LABEL(lbl), TRUE);
    }
}

void
xfmedia_mainwin_set_pause_buttons(XfMediaMainwin *mwin)
{
    gtk_container_remove(GTK_CONTAINER(mwin->playpause_btn),
            gtk_bin_get_child(GTK_BIN(mwin->playpause_btn)));
    gtk_container_add(GTK_CONTAINER(mwin->playpause_btn),
            mwin->pause_btn_img);
    gtk_container_remove(GTK_CONTAINER(mwin->fs_playpause_btn),
            gtk_bin_get_child(GTK_BIN(mwin->fs_playpause_btn)));
    gtk_container_add(GTK_CONTAINER(mwin->fs_playpause_btn),
            mwin->fs_pause_btn_img);
    if(mwin->tray_icon_playpause_btn && mwin->tray_icon_pause_img) {
        GtkWidget *lbl;
        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mwin->tray_icon_playpause_btn),
                mwin->tray_icon_pause_img);
        lbl = gtk_bin_get_child(GTK_BIN(mwin->tray_icon_playpause_btn));
        gtk_label_set_text(GTK_LABEL(lbl), _("_Pause"));
        gtk_label_set_use_underline(GTK_LABEL(lbl), TRUE);
    }
}

void
xfmedia_mainwin_destroy(XfMediaMainwin *mwin)
{    
    xfmedia_keybindings_cleanup();
    
    if(mwin->gtx)
        gtk_xine_stop(mwin->gtx);
    
    if(mwin->pos_id) {
        g_source_remove(mwin->pos_id);
        mwin->pos_id = 0;
    }
    
    if(mwin->plist) {
        DBG("destroying plist (%p)", mwin->plist);
        xfmedia_playlist_destroy(mwin->plist);
        mwin->plist = NULL;
    }
    
    if(mwin->tooltips) {
        gtk_object_sink(GTK_OBJECT(mwin->tooltips));
        mwin->tooltips = NULL;
    }
    
    DBG("closing extra stream and a/v drivers");
    if(mwin->extra_stream) {
        xine_close(mwin->extra_stream);
        xine_dispose(mwin->extra_stream);
        xine_close_audio_driver(gtk_xine_get_raw_xine_engine(mwin->gtx),
                mwin->extra_ap);
        xine_close_video_driver(gtk_xine_get_raw_xine_engine(mwin->gtx),
                mwin->extra_vp);
        mwin->extra_stream = NULL;
        mwin->extra_ap = NULL;
        mwin->extra_vp = NULL;
    }
    
    DBG("killing tray icon");
    if(mwin->tray_icon) {
        xfmedia_tray_icon_destroy(mwin->tray_icon);
        mwin->tray_icon = NULL;
    }
    
    if(mwin->fs_controls) {
        gtk_widget_destroy(mwin->fs_controls);
        mwin->fs_controls = NULL;
    }
    
    if(mwin->autohide_window) {
        gtk_widget_destroy(mwin->autohide_window);
        mwin->autohide_window = NULL;
    }
    
    g_free(mwin);
    mwin = NULL;
}
