/*
 *  xfmedia - simple gtk2 media player based on xine
 *
 *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gtk/gtkversion.h>
#if defined(GTK_DISABLE_DEPRECATED) && GTK_CHECK_VERSION(2, 4, 0)
#undef GTK_DISABLE_DEPRECATED
#endif
#include <gtk/gtk.h>

#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>

#include <xfmedia/xfmedia-settings.h>
#include "xfmedia-common.h"
#include "mainwin.h"
#include "xfmedia-keybindings.h"

#define _NET_WM_STATE_REMOVE 0
#define _NET_WM_STATE_ADD    1
#define _NET_WM_STATE_TOGGLE 2

struct PConfData
{
    GtkWidget *gtx;
    GtkWidget *cfgwin;
    GtkTreeSelection *treesel;
    const gchar *title;
};

struct SDData
{
    GtkTooltips *ttips;
    struct PConfData *ipcdata;
    struct PConfData *opcdata;
};

enum {
    OPLUGIN_LIST_ENABLED,
    OPLUGIN_LIST_NAME,
    OPLUGIN_LIST_OP,
    OPLUGIN_N_COLS
};

static gboolean sd_running = FALSE;

static gboolean
set_info_notloaded(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
        gpointer data)
{
    gtk_list_store_set(GTK_LIST_STORE(model), iter, PLAYLIST_COL_INFOLOADED,
            FALSE, -1);
    
    return FALSE;
}

static gboolean
titledisp_cb(GtkWidget *w, GdkEventFocus *evt, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    gchar *fmt;
    
    fmt = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    xfmedia_settings_set_string("/xfmedia/playlist/playlist_format", fmt);
    g_free(fmt);
    
    /* invalidate all the playlist info fields (expensive?) */
    gtk_tree_model_foreach(GTK_TREE_MODEL(mwin->plist->file_list),
            set_info_notloaded, NULL);
    
    return FALSE;
}

static void
autoadvance_cb(GtkToggleButton *tb, gpointer user_data)
{
    xfmedia_settings_set_bool("/xfmedia/playlist/auto_advance",
            gtk_toggle_button_get_active(tb));
}

static void
allws_cb(GtkToggleButton *tb, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    gboolean set;
    
    set = gtk_toggle_button_get_active(tb);
    if(set)
        gtk_window_stick(GTK_WINDOW(mwin->window));
    else
        gtk_window_unstick(GTK_WINDOW(mwin->window));
    
    xfmedia_settings_set_bool("/xfmedia/general/window_sticky", set);
}

static void
alwaysontop_cb(GtkToggleButton *tb, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    gboolean set;
    
    set = gtk_toggle_button_get_active(tb);
    gtk_window_set_keep_above(GTK_WINDOW(mwin->window), set);
    xfmedia_settings_set_bool("/xfmedia/general/window_above", set);
}

static void
trayicon_cb(GtkToggleButton *tb, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    gboolean set;
    
    set = gtk_toggle_button_get_active(tb);
    gtk_widget_set_sensitive(mwin->tray_mini_chk, set);
    if(set) {
        xfmedia_mainwin_create_systray_item(mwin);
        gtk_widget_show(mwin->tray_icon);
        if(mwin->is_iconified
                && xfmedia_settings_get_bool("/xfmedia/general/minimize_to_tray"))
        {
            gtk_widget_hide(mwin->window);
        }
    } else {
        gtk_window_set_skip_taskbar_hint(GTK_WINDOW(mwin->window), FALSE);
        gtk_widget_show(mwin->window);
        gtk_widget_destroy(mwin->tray_icon);
        mwin->tray_icon = NULL;
    }
    xfmedia_settings_set_bool("/xfmedia/general/show_tray_icon", set);
}

static void
trayiconmini_cb(GtkToggleButton *tb, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    gboolean set;
    
    set = gtk_toggle_button_get_active(tb);
    if(!set)
        gtk_window_set_skip_taskbar_hint(GTK_WINDOW(mwin->window), FALSE);
    else if(set && mwin->is_iconified) {
        gtk_window_set_skip_pager_hint(GTK_WINDOW(mwin->window), TRUE);
        gtk_widget_hide(mwin->window);
    }
    xfmedia_settings_set_bool("/xfmedia/general/minimize_to_tray", set);
}

static void
sd_destroy(struct SDData *sddata)
{
    gtk_object_sink(GTK_OBJECT(sddata->ttips));
    if(sddata->ipcdata)
        g_free(sddata->ipcdata);
    if(sddata->opcdata)
        g_free(sddata->opcdata);
    g_free(sddata);
    sd_running = FALSE;
}

#if 0

static void
plugin_setting_toggled_cb(GtkToggleButton *tb, gpointer user_data)
{
    XfMusicPluginSetting *ps = user_data;
    XfMusic *xfmusic = g_object_get_data(G_OBJECT(tb), "xfmusic");
    
    xfmedia_settings_set_bool(xfmusic, ps->key,
            gtk_toggle_button_get_active(tb));
}

static gboolean
plugin_setting_entry_cb(GtkWidget *w, GdkEventFocus *evt, gpointer user_data)
{
    XfMusicPluginSetting *ps = user_data;
    XfMusic *xfmusic = g_object_get_data(G_OBJECT(w), "xfmusic");
    gchar *entry_text;
    gint intval;
    gdouble floatval;
    
    entry_text = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
    if(!entry_text)
        return FALSE;
    
    switch(ps->type) {
        case XFMUSIC_SETTING_INT:
            intval = atoi(entry_text);
            xfmedia_settings_set_int(xfmusic, ps->key, intval);
            break;
        case XFMUSIC_SETTING_FLOAT:
            floatval = g_strtod(entry_text, NULL);
            xfmedia_settings_set_float(xfmusic, ps->key, floatval);
            break;
        case XFMUSIC_SETTING_STRING:
            xfmedia_settings_set_string(xfmusic, ps->key, entry_text);
            break;
    }
    
    g_free(entry_text);
    
    return FALSE;
}

static void
sd_plugin_conf_cb(GtkWidget *w, gpointer user_data)
{
    struct PConfData *pcdata = user_data;
    GtkTreeModel *model;
    GtkTreeIter itr;
    GList *settings, *l;
    XfMusicPlugin *op = NULL;
    GtkWidget *dlg, *topvbox, *lbl, *btn, *entry, *hbox, *frame;
    GtkTooltips *ttips;
    
    if(!gtk_tree_selection_get_selected(pcdata->treesel, &model, &itr))
        return;
    
    gtk_tree_model_get(GTK_TREE_MODEL(model), &itr, OPLUGIN_LIST_OP, &op, -1);
    if(!op)
        return;
    
    settings = xfmusic_plugin_query_settings(op);
    if(!settings) {
        xfce_info(_("The selected plugin has no user-configurable settings."));
        return;
    }
    
    dlg = gtk_dialog_new_with_buttons(pcdata->title, GTK_WINDOW(pcdata->cfgwin),
            GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
    gtk_dialog_set_default_response(GTK_DIALOG(dlg), GTK_RESPONSE_CLOSE);
    gtk_container_set_border_width(GTK_CONTAINER(dlg), BORDER);
    g_signal_connect_swapped(G_OBJECT(dlg), "destroy",
            G_CALLBACK(xfmusic_plugin_settings_free), settings);
    g_signal_connect(G_OBJECT(dlg), "response",
            G_CALLBACK(gtk_widget_destroy), NULL);
    
    frame = xfce_framebox_new(xfmusic_plugin_get_name(op), TRUE);
    gtk_widget_show(frame);
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dlg)->vbox), frame);
    
    topvbox = gtk_vbox_new(FALSE, BORDER);
    gtk_widget_show(topvbox);
    xfce_framebox_add(XFCE_FRAMEBOX(frame), topvbox);
    
    ttips = gtk_tooltips_new();
    
    for(l = settings; l; l = l->next) {
        XfMusicPluginSetting *ps = l->data;
        if(ps->type == XFMUSIC_SETTING_BOOL) {
            btn = gtk_check_button_new_with_label(ps->name);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
                    xfmedia_settings_get_bool(pcdata->xfmusic, ps->key));
            gtk_widget_show(btn);
            gtk_box_pack_start(GTK_BOX(topvbox), btn, FALSE, FALSE, 0);
            g_object_set_data(G_OBJECT(btn), "xfmusic", pcdata->xfmusic);
            g_signal_connect(G_OBJECT(btn), "toggled",
                    G_CALLBACK(plugin_setting_toggled_cb), ps);
            gtk_tooltips_set_tip(ttips, btn, ps->description, NULL);
        } else {
            gchar *lbl_str = g_strconcat(ps->name, ":", NULL);
            
            hbox = gtk_hbox_new(FALSE, BORDER);
            gtk_widget_show(hbox);
            gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
            
            lbl = gtk_label_new(lbl_str);
            gtk_widget_show(lbl);
            gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
            g_free(lbl_str);
            
            entry = gtk_entry_new();
            if(ps->type == XFMUSIC_SETTING_STRING) {
                const gchar *str;
                str = xfmedia_settings_get_string(pcdata->xfmusic, ps->key);
                if(str)
                    gtk_entry_set_text(GTK_ENTRY(entry), str);
            } else if(ps->type == XFMUSIC_SETTING_INT) {
                gchar tmp[64];
                gint val = xfmedia_settings_get_int(pcdata->xfmusic, ps->key);
                g_snprintf(tmp, 64, "%d", val);
                gtk_entry_set_text(GTK_ENTRY(entry), tmp);
            } else if(ps->type == XFMUSIC_SETTING_FLOAT) {
                gchar tmp[64];
                gdouble val = xfmedia_settings_get_float(pcdata->xfmusic, ps->key);
                g_snprintf(tmp, 64, "%.4f", val);
                gtk_entry_set_text(GTK_ENTRY(entry), tmp);
            }
            gtk_widget_show(entry);
            gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
            g_object_set_data(G_OBJECT(entry), "xfmusic", pcdata->xfmusic);
            g_signal_connect(G_OBJECT(entry), "focus-out-event",
                    G_CALLBACK(plugin_setting_entry_cb), ps);
            gtk_tooltips_set_tip(ttips, entry, ps->description, NULL);
        }
    }
    
    gtk_widget_show(dlg);
}

static gboolean
op_treeview_button_cb(GtkWidget *w, GdkEventButton *evt, gpointer user_data)
{
    if(evt->type == GDK_2BUTTON_PRESS)
        sd_plugin_conf_cb(NULL, user_data);
    
    return FALSE;
}

static void
sd_plugin_about_cb(GtkWidget *w, gpointer user_data)
{
    struct PConfData *pcdata = user_data;
    GtkTreeModel *model;
    GtkTreeIter itr;
    XfMusicPlugin *pl = NULL;
    const gchar *name, *desc, *author, *version, *website;
    GtkWidget *dlg, *topvbox, *lbl, *hbox;
    GtkSizeGroup *sg;
    
    if(!gtk_tree_selection_get_selected(pcdata->treesel, &model, &itr))
        return;
    
    gtk_tree_model_get(GTK_TREE_MODEL(model), &itr, OPLUGIN_LIST_OP, &pl, -1);
    if(!pl)
        return;
    
    name = xfmusic_plugin_get_name(pl);
    desc = xfmusic_plugin_get_description(pl);
    author = xfmusic_plugin_get_author(pl);
    version = xfmusic_plugin_get_version(pl);
    website = xfmusic_plugin_get_website(pl);
    
    dlg = gtk_dialog_new_with_buttons(pcdata->title, GTK_WINDOW(pcdata->cfgwin),
            GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
    gtk_dialog_set_default_response(GTK_DIALOG(dlg), GTK_RESPONSE_CLOSE);
    gtk_container_set_border_width(GTK_CONTAINER(dlg), BORDER);
    g_signal_connect(G_OBJECT(dlg), "response", G_CALLBACK(gtk_widget_destroy), NULL);
    
    topvbox = GTK_DIALOG(dlg)->vbox;
    gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
    
    sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new(_("Name:"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    gtk_size_group_add_widget(sg, lbl);
    
    lbl = gtk_label_new(name ? name : _("Unknown"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new(_("Version:"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    gtk_size_group_add_widget(sg, lbl);
    
    lbl = gtk_label_new(version ? version : _("Unknown"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new(_("Description:"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.0);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    gtk_size_group_add_widget(sg, lbl);
    
    lbl = gtk_label_new(desc ? desc : _("Unknown"));
    gtk_label_set_line_wrap(GTK_LABEL(lbl), TRUE);
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new(_("Author:"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    gtk_size_group_add_widget(sg, lbl);
    
    lbl = gtk_label_new(author ? author : _("Unknown"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new(_("Website:"));
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    gtk_size_group_add_widget(sg, lbl);
    
    lbl = gtk_label_new(website ? website : _("Unknown"));
    gtk_label_set_selectable(GTK_LABEL(lbl), TRUE);
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    gtk_widget_show(dlg);
}    

static void
sd_toggle_op_cb(GtkCellRendererToggle *crt, gchar *arg1, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    gboolean cur_state;
    GtkTreeIter itr;
    GtkListStore *ls;
    GList *oplugins;
    XfMusicPlugin *op;
    
    cur_state = gtk_cell_renderer_toggle_get_active(crt);
    if(cur_state)
        return;
    
    ls = GTK_LIST_STORE(gtk_tree_view_get_model(mwin->op_list));
    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(ls), &itr);
    while(gtk_list_store_iter_is_valid(ls, &itr)) {
        gtk_list_store_set(ls, &itr, OPLUGIN_LIST_ENABLED, FALSE, -1);
        gtk_tree_model_iter_next(GTK_TREE_MODEL(ls), &itr);
    }
    
    if(gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(ls), &itr, arg1)) {
        oplugins = xfmusic_get_plugins(mwin->xfmusic, XFMUSIC_PLUGIN_TYPE_OUTPUT);
        op = g_list_nth_data(oplugins, atoi(arg1));
        if(op) {
            gtk_list_store_set(ls, &itr, OPLUGIN_LIST_ENABLED, TRUE, -1);
            xfmusic_set_output_plugin(mwin->xfmusic, op);
            xfmedia_settings_set_string(mwin->xfmusic,
                    "/libxfmusic4/audio/output/cur_plugin", op->id);
        }
    }
}

static GtkTreeView *
sd_add_oplugin_treeview(XfMediaMainwin *mwin, GtkWidget *topvbox)
{
    GtkWidget *treeview, *sw;
    GtkListStore *ls;
    GtkCellRenderer *render;
    GtkTreeViewColumn *col;
    GtkTreeIter itr;
    GList *plugins, *l;
    XfMusicPlugin *cur_op;
    const gchar *name;
    
    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER,
            GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
            GTK_SHADOW_ETCHED_IN);
    gtk_widget_show(sw);
    gtk_box_pack_start(GTK_BOX(topvbox), sw, TRUE, TRUE, 0);
    
    ls = gtk_list_store_new(OPLUGIN_N_COLS, G_TYPE_BOOLEAN, G_TYPE_STRING,
            G_TYPE_POINTER);
    
    cur_op = xfmusic_get_current_output_plugin(mwin->xfmusic);
    plugins = xfmusic_get_plugins(mwin->xfmusic, XFMUSIC_PLUGIN_TYPE_OUTPUT);
    for(l = plugins; l; l = l->next) {
        name = xfmusic_plugin_get_name((XfMusicPlugin *)l->data);
        gtk_list_store_append(ls, &itr);
        gtk_list_store_set(ls, &itr, OPLUGIN_LIST_ENABLED, (cur_op == l->data),
                OPLUGIN_LIST_NAME, name, OPLUGIN_LIST_OP, l->data, -1);
    }        
    
    mwin->op_list = GTK_TREE_VIEW(treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ls)));
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
    
    render = gtk_cell_renderer_toggle_new();
    col = gtk_tree_view_column_new_with_attributes("enable", render, "active",
            OPLUGIN_LIST_ENABLED, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
    g_signal_connect(G_OBJECT(render), "toggled",
            G_CALLBACK(sd_toggle_op_cb), mwin);
    
    render = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes("plugin name", render,
            "text", OPLUGIN_LIST_NAME, NULL);
    g_object_set(G_OBJECT(col), "expand", TRUE, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
    
    gtk_widget_show(treeview);
    gtk_container_add(GTK_CONTAINER(sw), treeview);
    
    return GTK_TREE_VIEW(treeview);
}

#endif

static void
soea_toggle_cb(GtkToggleButton *tb, gpointer user_data)
{
    xfmedia_settings_set_bool("/xfmedia/playlist/save_on_exit_always",
            !gtk_toggle_button_get_active(tb));
}

static void
dock_autohide_cb(GtkToggleButton *tb, gpointer user_data)
{
    xfmedia_settings_set_bool("/xfmedia/general/dock_autohide",
            gtk_toggle_button_get_active(tb));
}

static void
metadata_load_change_cb(GtkToggleButton *tb, gpointer user_data)
{
    guint setting = GPOINTER_TO_UINT(user_data);
    
    if(gtk_toggle_button_get_active(tb))
        xfmedia_settings_set_int("/xfmedia/playlist/metadata_load_impetus", setting);
}

static void
audio_vis_changed_cb(GtkMenuItem *mi, gpointer user_data)
{
    XfMediaMainwin *mwin = user_data;
    GtkLabel *lbl = GTK_LABEL(gtk_bin_get_child(GTK_BIN(mi)));
    const gchar *vis_name = gtk_label_get_text(lbl);
    
    xfmedia_settings_set_string("/xfmedia/general/selected_vis", vis_name);
    
    if(xfmedia_settings_get_bool("/xfmedia/general/show_audio_vis")
            && gtk_xine_get_status(mwin->gtx) == XINE_STATUS_PLAY
            && !gtk_xine_get_stream_info(mwin->gtx, XINE_STREAM_INFO_HAS_VIDEO))
    {
        gtk_xine_set_vis(mwin->gtx, vis_name);
    }
}

void
xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
{
    static GtkWidget *win = NULL;
    GtkWidget *topvbox, *vbox, *hbox, *btn, *frame, *entry, *lbl;
    GtkWidget *notebook, *menu, *mi, *cb;
    GtkTooltips *ttips;
    struct SDData *sddata;
    GList *vis_plugins;
    gint selected_vis_index = 0;
    //GtkTreeView *treeview;
    //GtkTreeSelection *treesel;
    
    if(sd_running && win) {
        gtk_window_present(GTK_WINDOW(win));
        return;
    }
    
    sddata = g_new0(struct SDData, 1);
    sddata->ipcdata = g_new(struct PConfData, 1);
    sddata->opcdata = g_new(struct PConfData, 1);
    
    sddata->ttips = ttips = gtk_tooltips_new();
    
    win = gtk_dialog_new_with_buttons(_("Xfmedia Preferences"),
            GTK_WINDOW(mwin->window),
            GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
    gtk_dialog_set_default_response(GTK_DIALOG(win), GTK_RESPONSE_CLOSE);
    gtk_container_set_border_width(GTK_CONTAINER(win), BORDER);
    g_signal_connect_swapped(G_OBJECT(win), "destroy",
            G_CALLBACK(sd_destroy), sddata);
    g_signal_connect(G_OBJECT(win), "response",
            G_CALLBACK(gtk_widget_destroy), NULL);
    
    notebook = gtk_notebook_new();
    gtk_widget_show(notebook);
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(win)->vbox), notebook);
    
    
    /* options tab */
    
    topvbox = gtk_vbox_new(FALSE, BORDER);
    gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
    gtk_widget_show(topvbox);
    
    frame = xfce_framebox_new(_("Playlist"), TRUE);
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
    
    vbox = gtk_vbox_new(FALSE, BORDER/2);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
    gtk_widget_show(vbox);
    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new_with_mnemonic(_("_Title Display:"));
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(entry),
            xfmedia_settings_get_string("/xfmedia/playlist/playlist_format"));
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
    gtk_tooltips_set_tip(ttips, entry,
            _("This is a printf-like format string controlling the display of" \
              " the playlist.  Xfmedia will replace the following symbols:\n" \
              "\t%p: Artist/Performer\n" \
              "\t%t: Track Title\n" \
              "\t%a: Album Name\n" \
              "\t%T: Track Number\n" \
              "\t%g: Genre\n" \
              "\t%y: Year\n" \
              "\t%f: Filename\n" \
              "\t%%: a percent sign"), NULL);
    g_signal_connect(G_OBJECT(entry), "focus-out-event",
            G_CALLBACK(titledisp_cb), mwin);
    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), entry);
    
    btn = gtk_check_button_new_with_mnemonic(_("_Advance playlist when track ends"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            xfmedia_settings_get_bool("/xfmedia/playlist/auto_advance"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled",
            G_CALLBACK(autoadvance_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn,
            _("When a track ends, Xfmedia can automatically advance to the next track in the playlist and continue playback."), NULL);            
    
    btn = gtk_check_button_new_with_mnemonic(_("_Don't save playlist on exit for one-shot invocations"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            !xfmedia_settings_get_bool("/xfmedia/playlist/save_on_exit_always"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled",
            G_CALLBACK(soea_toggle_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn,
            _("Only automatically save the playlist during interactive use; i.e., don't save the playlist when Xfmedia was invoked from the commandline with a short list of definite arguments.  This option needs to be named better."), NULL);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    
    lbl = gtk_label_new(_("Load metadata:"));
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    btn = gtk_radio_button_new_with_mnemonic(NULL, _("On _scroll"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            (xfmedia_settings_get_int("/xfmedia/playlist/metadata_load_impetus") == 0));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled",
            G_CALLBACK(metadata_load_change_cb), GUINT_TO_POINTER(0));
    gtk_tooltips_set_tip(ttips, btn,
            _("Load file metadata for entries in the playlist when they are visible."), NULL);
    
    btn = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(btn),
            _("On _play"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            (xfmedia_settings_get_int("/xfmedia/playlist/metadata_load_impetus") == 1));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled",
            G_CALLBACK(metadata_load_change_cb), GUINT_TO_POINTER(1));
    gtk_tooltips_set_tip(ttips, btn,
            _("Load file metadata for entries in the playlist only when playing them."), NULL);
    
    btn = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(btn),
            _("_Never"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            (xfmedia_settings_get_int("/xfmedia/playlist/metadata_load_impetus") == 2));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled",
            G_CALLBACK(metadata_load_change_cb), GUINT_TO_POINTER(2));
    gtk_tooltips_set_tip(ttips, btn,
            _("Never load file metadata for entries in the playlist."), NULL);
    
    frame = xfce_framebox_new(_("Window"), TRUE);
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
    
    vbox = gtk_vbox_new(FALSE, BORDER/2);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
    gtk_widget_show(vbox);
    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
    
    btn = gtk_check_button_new_with_mnemonic(_("Show Xfmedia on all _workspaces"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            xfmedia_settings_get_bool("/xfmedia/general/window_sticky"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(allws_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Sets Xfmedia \"sticky\" such that it appears on all workspaces (virtual desktops)."), NULL);
    
    btn = gtk_check_button_new_with_mnemonic(_("_Keep Xfmedia above other windows"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            xfmedia_settings_get_bool("/xfmedia/general/window_above"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(alwaysontop_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn,
            _("Sets the Xfmedia window to \"Always-on-Top\" status."), NULL);
            
    btn = gtk_check_button_new_with_mnemonic(_("_Dock and auto-hide Xfmedia at screen edges"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            xfmedia_settings_get_bool("/xfmedia/general/dock_autohide"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(dock_autohide_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn,
            _("When the Xfmedia window is dragged to a screen edge, it will auto-hide itself and reappear when you move the mouse to the edge of the screen."), NULL);
    
    frame = xfce_framebox_new(_("Tray Icon"), TRUE);
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
    
    vbox = gtk_vbox_new(FALSE, BORDER/2);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
    gtk_widget_show(vbox);
    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
    
    btn = gtk_check_button_new_with_mnemonic(_("Show _tray icon"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            xfmedia_settings_get_bool("/xfmedia/general/show_tray_icon"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(trayicon_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Displays an Xfmedia icon in the system notification area."), NULL);
    
    mwin->tray_mini_chk = btn = gtk_check_button_new_with_mnemonic(_("_Minimize Xfmedia to tray icon"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
            xfmedia_settings_get_bool("/xfmedia/general/minimize_to_tray"));
    gtk_widget_set_sensitive(btn,
            xfmedia_settings_get_bool("/xfmedia/general/show_tray_icon"));
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(trayiconmini_cb), mwin);
    gtk_tooltips_set_tip(ttips, btn, _("Minimizes Xfmedia to the tray icon rather than to the taskbar or iconbox."), NULL);
    
    lbl = gtk_label_new_with_mnemonic(_("O_ptions"));
    gtk_widget_show(lbl);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
    
    
    /* vis settings tab */
    
    topvbox = gtk_vbox_new(FALSE, BORDER);
    gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
    gtk_widget_show(topvbox);
    
    frame = xfce_framebox_new(_("Selected Visualization"), TRUE);
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, BORDER/2);
    gtk_widget_show(hbox);
    xfce_framebox_add(XFCE_FRAMEBOX(frame), hbox);
    
    menu = gtk_menu_new();
    gtk_widget_show(menu);
    
    vis_plugins = gtk_xine_list_post_plugins_typed(mwin->gtx,
            XINE_POST_TYPE_AUDIO_VISUALIZATION);
    if(vis_plugins) {
        GList *l;
        const gchar *selected_vis =
                xfmedia_settings_get_string("/xfmedia/general/selected_vis");
        for(l = vis_plugins; l; l = l->next) {
            if(!g_ascii_strcasecmp((const gchar *)l->data, selected_vis))
                selected_vis_index = g_list_index(vis_plugins, l->data);
            mi = gtk_menu_item_new_with_label((const gchar *)l->data);
            gtk_widget_show(mi);
            gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
            g_signal_connect(G_OBJECT(mi), "activate",
                    G_CALLBACK(audio_vis_changed_cb), mwin);
        }
        g_list_free(vis_plugins);
    } else {
        mi = gtk_menu_item_new_with_label(_("(none available)"));
        gtk_widget_show(mi);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    }
    
    cb = gtk_option_menu_new();
    gtk_option_menu_set_menu(GTK_OPTION_MENU(cb), menu);
    gtk_option_menu_set_history(GTK_OPTION_MENU(cb), selected_vis_index);
    gtk_widget_show(cb);
    gtk_box_pack_start(GTK_BOX(hbox), cb, TRUE, TRUE, 0);
    
    frame = xfce_framebox_new(_("Visualization Options"), TRUE);
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
    
    vbox = gtk_vbox_new(FALSE, BORDER/2);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
    gtk_widget_show(vbox);
    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
    
    lbl = gtk_label_new("(not yet implemented)");
    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(vbox), lbl, FALSE, FALSE, 0);
    
    
    lbl = gtk_label_new_with_mnemonic(_("_Audio Visualization"));
    gtk_widget_show(lbl);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
    
    /* keybindings editor */
    topvbox = xfmedia_keybindings_create_editor(mwin, win);
    lbl = gtk_label_new_with_mnemonic(_("_Keybindings"));
    gtk_widget_show(lbl);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
    
    
#if 0
    /* output plugins tab */
    
    topvbox = gtk_vbox_new(FALSE, BORDER);
    gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
    gtk_widget_show(topvbox);
    
    treeview = sd_add_oplugin_treeview(mwin, topvbox);
    treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
    sddata->opcdata->xfmusic = mwin->xfmusic;
    sddata->opcdata->cfgwin = win;
    sddata->opcdata->treesel = treesel;
    sddata->opcdata->title = _("Output Plugin Settings");
    
    g_signal_connect(G_OBJECT(treeview), "button-press-event",
            G_CALLBACK(op_treeview_button_cb), sddata->opcdata);
    
    hbox = gtk_hbox_new(FALSE, BORDER);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
    
    btn = xfmusic_custom_button_new(_("_Settings"), GTK_STOCK_PREFERENCES);
    //gtk_widget_set_sensitive(btn, FALSE);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(sd_plugin_conf_cb), sddata->opcdata);
    
    btn = xfmusic_custom_button_new_from_pixdata(_("_About"), &about_icon);
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(btn), "clicked",
            G_CALLBACK(sd_plugin_about_cb), sddata->opcdata);
    
    lbl = gtk_label_new_with_mnemonic(_("_Output Plugins"));
    gtk_widget_show(lbl);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
    
#endif
    
    sd_running = TRUE;
    gtk_widget_show(win);

}
