/*
 *  xfmedia - simple gtk2 media player based on xine
 *
 *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include <libxfcegui4/netk-trayicon.h>

#include "trayicon.h"
#include "xfmedia-common.h"

GtkWidget *
xfmedia_tray_icon_create(GdkScreen *gscreen)
{
    GtkWidget *tray_icon, *evtbox, *img;
    GdkPixbuf *pix, *tmp;
    gint iw, ih;
    
    tray_icon = netk_tray_icon_new(GDK_SCREEN_XSCREEN(gscreen));
    
    evtbox = gtk_event_box_new();
    gtk_widget_add_events(evtbox, GDK_BUTTON_PRESS);
    gtk_widget_show(evtbox);
    gtk_container_add(GTK_CONTAINER(tray_icon), evtbox);
    
    gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &iw, &ih);
    pix = xfce_icon_theme_load(icon_theme, "xfmedia", iw);
    if(!pix) {
        tmp = gdk_pixbuf_from_pixdata(&xfmedia_icon, TRUE, NULL);
        if(tmp && gdk_pixbuf_get_width(tmp) == iw && gdk_pixbuf_get_height(tmp) == ih)
            pix = tmp;
        else if(tmp) {
            pix = gdk_pixbuf_scale_simple(tmp, iw, ih, GDK_INTERP_BILINEAR);
            g_object_unref(G_OBJECT(tmp));
        } else {
            pix = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, iw, ih);
            gdk_pixbuf_fill(pix, 0);
        }
    }
    
    img = gtk_image_new_from_pixbuf(pix);
    g_object_unref(G_OBJECT(pix));
    gtk_widget_show(img);
    gtk_container_add(GTK_CONTAINER(evtbox), img);
    
    return tray_icon;
}

void
xfmedia_tray_icon_destroy(GtkWidget *tray_icon)
{
    g_return_if_fail(NETK_IS_TRAY_ICON(tray_icon));
    
    gtk_widget_destroy(tray_icon);
}
